@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2004, 2008, 2009, 2010, 2011
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2004, 2008, 2009, 2010, 2011,
+ *               2013
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,17 +42,15 @@
  *
  */
 
-/* $Id: interposition.h,v 1.82 2011/05/18 13:48:45 karls Exp $ */
+/* $Id: interposition.h,v 1.86 2013/10/27 15:24:41 karls Exp $ */
 
 #ifndef _INTERPOSITION_H_
 #define _INTERPOSITION_H_
 
-#if !BAREFOOTD
 #include "symbols.h"
-#endif /* !BAREFOOTD */
 
 typedef enum { pid = 0, thread } which_id_t;
-struct socks_id_t {
+typedef struct socks_id_t {
    which_id_t        whichid;
    union {
       pid_t          pid;
@@ -61,24 +60,24 @@ struct socks_id_t {
    } id;
 
    struct socks_id_t *next;
-};
+} socks_id_t;
 
-struct libsymbol_t {
+typedef struct {
    char  *symbol;         /* name of the symbol.         */
    char  *library;        /* library symbol is in.       */
    void  *handle;         /* handle to the library.      */
    void  *function;       /* the bound symbol.           */
 
-   struct socks_id_t *dosyscall; /*
-                                  * if this value is not set, the corresponding
-                                  * syscall should be used for the given id.
-                                  * This is for cases where we are unable to
-                                  * base the decision concerning whether the
-                                  * function should resolve to a R*() function
-                                  * or a syscall in other ways.
-                                  */
+   socks_id_t *dosyscall; /*
+                           * if this value is not set, the corresponding
+                           * syscall should be used for the given id.
+                           * This is for cases where we are unable to
+                           * base the decision concerning whether the
+                           * function should resolve to a R*() function
+                           * or a syscall in other ways.
+                           */
 
-};
+} libsymbol_t;
 
 #if SOCKS_CLIENT
 
@@ -110,16 +109,8 @@ socks_issyscall(const int s, const char *name);
  * Returns true if so, false otherwise.
  */
 
-
-#else /* !SOCKS_CLIENT */
-#define socks_syscall_start(s)
-#define socks_syscall_end(s)
-#endif /* !SOCKS_CLIENT */
-
-#if SOCKS_CLIENT
-
-struct socks_id_t *
-socks_whoami(struct socks_id_t *id);
+socks_id_t *
+socks_whoami(socks_id_t *id);
 /*
  * Returns a unique id identifying the calling thread or process,
  * depending on whether the process is threaded or not.
@@ -127,6 +118,22 @@ socks_whoami(struct socks_id_t *id);
  * Returns "id".
  */
 
+
+#else /* !SOCKS_CLIENT */
+
+#define socks_syscall_start(s)
+#define socks_syscall_end(s)
+
+#define socks_whoami(_id)                                                      \
+do {                                                                           \
+   (_id)->whichid = pid;                                                       \
+   (_id)->id.pid  = sockscf.state.pid;                                         \
+   (_id)->next    = NULL;                                                      \
+} while (/* CONSTCOND */ 0)
+
+#endif /* !SOCKS_CLIENT */
+
+
 int
 socks_shouldcallasnative(const char *functionname);
 /*
@@ -153,12 +160,6 @@ socks_markasnormal(const char *functionname);
  * used when resolving "functionname".
  */
 
-#else /* ! SOCKS_CLIENT */
-
-#define socks_shouldcallasnative(name)  (0) /* not needed in the server. */
-
-#endif /* !SOCKS_CLIENT */
-
 void *
 symbolfunction(const char *symbol);
 /*
