@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2005, 2008, 2009, 2010,
- *               2011, 2012, 2013
+ *               2011, 2012, 2013, 2014
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: config.c,v 1.464 2013/10/27 15:24:42 karls Exp $";
+"$Id: config.c,v 1.464.4.2 2014/08/15 18:16:40 karls Exp $";
 
 void
 genericinit(void)
@@ -1205,7 +1205,7 @@ socks_routesetup(control, data, route, emsg, emsglen)
 
    if (control_type == SOCK_DGRAM) { /* only support udp now. */
       if (socks_rebind(control,
-                       control_type == SOCK_STREAM ? SOCKS_TCP : SOCK_DGRAM,
+                       control_type == SOCK_STREAM ? SOCKS_TCP : SOCKS_UDP,
                        &controladdr,
                        &route->rdr_from,
                        emsg,
@@ -1223,7 +1223,7 @@ socks_routesetup(control, data, route, emsg, emsglen)
 
    if (data_type == SOCK_DGRAM) { /* only support udp now. */
       if (socks_rebind(data,
-                       data_type == SOCK_STREAM ? SOCKS_TCP : SOCK_DGRAM,
+                       data_type == SOCK_STREAM ? SOCKS_TCP : SOCKS_UDP,
                        &dataaddr,
                        &route->rdr_from,
                        emsg,
