@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2003, 2004, 2005, 2008, 2009,
- *               2010, 2011, 2012, 2013
+ *               2010, 2011, 2012, 2013, 2014
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: connectchild.c,v 1.397 2013/10/27 15:24:42 karls Exp $";
+"$Id: connectchild.c,v 1.397.4.3 2014/08/15 18:16:40 karls Exp $";
 
 /*
  * This sets things up for performing a non-blocking connect for the client.
@@ -1445,8 +1445,6 @@ sigio(sig, sip, scp)
          };
 
          signalslog(LOG_DEBUG, msgv);
-
-         local = NULL;
       }
 
       len = sizeof(*remote);
@@ -1471,11 +1469,9 @@ sigio(sig, sip, scp)
          };
 
          signalslog(LOG_DEBUG, msgv);
-
-         remote = NULL;
       }
 
-        childres.packet.req.auth
+      childres.packet.req.auth
       = childres.packet.res.auth
       = &childres.packet.state.auth;
 
